@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /*
@@ -32,22 +32,23 @@
  * considered as unsigned chars
  */
 
-int memcmp(const void *b1, const void *b2, size_t len)
+int
+memcmp(const void *b1, const void *b2, size_t len)
 {
-	unsigned char *a, *b;
-	size_t i;
+    unsigned char *a, *b;
+    size_t i;
 
-	a = (unsigned char *)b1;
-	b = (unsigned char *)b2;
-	for(i = 0; i < len; i++) {
-		if (*a != *b) {
-			 /* strings differ */
-			return (*a < *b) ? -1 : 1;
-		}
-		a++;
-		b++;
-	}
+    a = (unsigned char *)b1;
+    b = (unsigned char *)b2;
+    for (i = 0; i < len; i++) {
+        if (*a != *b) {
+            /* strings differ */
+            return (*a < *b) ? -1 : 1;
+        }
+        a++;
+        b++;
+    }
 
-	/* strings match */
-	return 0;
+    /* strings match */
+    return 0;
 }
