@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /*
@@ -28,21 +28,9 @@
 #ifndef _JBIG2_OS_TYPES_H
 #define _JBIG2_OS_TYPES_H
 
-#if defined(__CYGWIN__) && !defined(HAVE_STDINT_H)
-# include <sys/types.h>
-# if defined(OLD_CYGWIN_SYS_TYPES)
-/*
- * Old versions of Cygwin have no stdint.h but define "MS types". Some of
- * them conflict with a standard type emulation provided by config_types.h
- * so we do a fixup here.
- */
-typedef u_int8_t uint8_t;
-typedef u_int16_t uint16_t;
-typedef u_int32_t uint32_t;
-#endif
-#elif defined(HAVE_CONFIG_H)
+#if defined(HAVE_CONFIG_H)
 # include "config_types.h"
-#elif defined(_WIN32) || defined(__WIN32__)
+#elif defined(_WIN32)
 # include "config_win32.h"
 #elif defined (STD_INT_USE_SYS_TYPES_H)
 # include <sys/types.h>
@@ -52,23 +40,8 @@ typedef u_int32_t uint32_t;
 # include <sys/inttypes.h>
 #elif defined (STD_INT_USE_SYS_INT_TYPES_H)
 # include <sys/int_types.h>
-#elif !defined(HAVE_STDINT_H)
-typedef unsigned char uint8_t;
-typedef unsigned short uint16_t;
-typedef unsigned int uint32_t;
-typedef signed char int8_t;
-typedef signed short int16_t;
-typedef signed int int32_t;
-#endif
-
-#if defined(HAVE_STDINT_H) || defined(__MACOS__)
+#else
 # include <stdint.h>
-#elif defined(__VMS) || defined(__osf__)
-# include <inttypes.h>
-#endif
-
-#ifdef __hpux
-#include <sys/_inttypes.h>
 #endif
 
 #endif /* _JBIG2_OS_TYPES_H */
